function [F,XL,YL,XR,YR,dlopt,dropt] = RANSAC_Feature_Matching(xl, yl, xr, yr, N, Dmax, metodo)
% [F,XL,YL,XR,YR,dlopt,dropt] = RANSAC_Feature_Matching(xl, yl, xr, yr, N, Dmax, metodo)
% Esta funo faz a depurao de listas de pontos homlogos usando a matriz
% fundamental como critrio. Retorna os pares de pontos sobreviventes e a
% correspondente matriz fundamental
% 
% Entradas da Funcao:
% xl - vetor com as coordenadas x dos pontos, na imagem da esquerda
% yl - vetor com as coordenadas y dos pontos, na imagem da esquerda
% xr - vetor com as coordenadas x dos pontos, na imagem da direita
% yr - vetor com as coordenadas y dos pontos, na imagem da direita
% N  - nmero de tentativas
% Dmax  - limiar de aceitao (em pixels) 
% metodo - String referente ao Metodo usado para calcular F 
%          'oito-pontos' - Algoritmo de 8 pontos
%          'luong-lin' - Mtodo linear de Luong
%          'luong-nonlin' - Mtodo no-linear de Luong
%          'hartley' - Mtodo de Hartley
%
% Sadas da Funcao:
% F - Matriz Fundamental 
% XL - vetor com as coordenadas x dos pontos, na imagem da esquerda
% YL - vetor com as coordenadas y dos pontos, na imagem da esquerda
% XR - vetor com as coordenadas x dos pontos, na imagem da direita
% YR - vetor com as coordenadas y dos pontos, na imagem da direita
% dlopt,dropt - distncias mdias s retas epipolares nas imagens
%       da esquerda e da direta.
%
% Raul Queiroz Feitosa - Junho/2008


%==========================================================================
% Prepara os conjuntos de pontos para clculo da primeira estimativa
%==========================================================================

Arranjos=[];
for i=1:N,
    Permutacao=randperm(length(xl));
    Arranjos=[Arranjos;Permutacao(1:9)];
end

if N>size(Arranjos,1),
    N=size(Arranjos,1);
end
indicek=randperm(size(Arranjos,1));
Arranjos=Arranjos(indicek(1:N),:);
%==========================================================================

%==========================================================================
% Loop para N Estimativas de F
%==========================================================================
pl=[xl(:)';yl(:)';ones(1,length(xr))];
pr=[xr(:)';yr(:)';ones(1,length(xr))];
Sopt=[];
h=waitbar(0,['RANSAC calculando F (mtodo = ' metodo ')' ]);
for contador=1:N,
    indicek=Arranjos(contador,:);
    
    xlk=xl(indicek);
    ylk=yl(indicek);
    xrk=xr(indicek);
    yrk=yr(indicek);
    
    plk=[xlk(:)';ylk(:)';ones(1,length(xrk))];
    prk=[xrk(:)';yrk(:)';ones(1,length(xrk))];    

    switch lower(metodo)
        case {'oito-pontos'}
            F = oito_pontos(xlk,ylk,xrk,yrk);
        case {'luong-lin'}
            F = luong_lin(xlk,ylk,xrk,yrk);
        case {'luong-nonlin'}
            F = luong_nonlin(xlk,ylk,xrk,yrk);
        case {'hartley'}
            F = hartley(xlk,ylk,xrk,yrk);
        otherwise
            error('Mtodo de Estimativa de F desconhecido!');
    end
    
    plFpr=sum(pl.*(F*pr));
    
    [dl2,dr2]=Calcula_Distancia_Linha_Epipolar(pl,F,pr);
    
    indice_prox=find((dl2 < (Dmax^2)) & (dr2 < (Dmax^2)));
    
    if (length(indice_prox)>length(Sopt)) ;
        Sopt=indice_prox;        
        Sk=indicek;
        Fopt=F;
        dl2opt=dl2;dr2opt=dr2;
    end

    waitbar(contador/N,h,['RANSAC calculando F (mtodo = ' metodo ') ' num2str(contador) '/' num2str(N)]);

end
close(h)
%==========================================================================

XL=xl(Sopt);
YL=yl(Sopt);
XR=xr(Sopt);
YR=yr(Sopt);

%==========================================================================
% Recalcula considerando todos os pontos "prximos"
%==========================================================================
switch lower(metodo)
    case {'oito-pontos'}
        F = oito_pontos(XL,YL,XR,YR);
    case {'luong-lin'}
        F = luong_lin(XL,YL,XR,YR);
    case {'luong-nonlin'}
        F = luong_nonlin(XL,YL,XR,YR);
    case {'hartley'}
        F = hartley(XL,YL,XR,YR);
    otherwise
        error('Mtodo de Estimativa de F desconhecido!');
end
%==========================================================================

%==========================================================================
% Calcula distncias mdia aos pontos "prximos" usando Fopt (calculado a
% partir dos melhores 9 pontos APENAS
%==========================================================================
pL=[XL';YL';ones(1,length(YL))];
pR=[XR';YR';ones(1,length(YL))];

[dl2opt,dr2opt]=Calcula_Distancia_Linha_Epipolar(pL,Fopt,pR);
dopt=mean(sqrt([dl2opt;dr2opt]),2);
%==========================================================================


%==========================================================================
% Calcula distncias mdia aos pontos "prximos" usando F (calculado a
% partir de TODOS os pontos "prximos"
%==========================================================================
[dl2,dr2]=Calcula_Distancia_Linha_Epipolar(pL,F,pR);
%==========================================================================
d=mean(sqrt([dl2;dr2]),2);

ds=mean(sqrt([dl2opt;dr2opt;dl2;dr2]),2);
if max(dopt)< max(d)
    F=Fopt;
    dlopt=mean(sqrt(dl2opt));
    dropt=mean(sqrt(dr2opt));
else
    dlopt=mean(sqrt(dl2));
    dropt=mean(sqrt(dr2));
end
    
return



